@@ -1,11 +1,9 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 {-# LANGUAGE OverloadedStrings #-}
 
-import Test.Hspec.Monadic
-import Test.Hspec.HUnit ()
+import Test.Hspec
 import Test.Hspec.QuickCheck (prop)
 import Test.QuickCheck (Arbitrary (..))
-import Test.HUnit
 
 import Codec.Zlib
 import Codec.Compression.Zlib
@@ -13,8 +11,10 @@ import qualified Codec.Compression.GZip as Gzip
 import qualified Data.ByteString as S
 import qualified Data.ByteString.Char8 as S8
 import qualified Data.ByteString.Lazy as L
+import qualified Data.ByteString.Lazy.Internal as LI
 import Control.Monad (foldM, forM_, forM)
 import System.IO.Unsafe (unsafePerformIO)
+import qualified Codec.Compression.Zlib.Raw as Raw
 
 decompress' :: L.ByteString -> L.ByteString
 decompress' gziped = unsafePerformIO $ do
@@ -84,7 +84,7 @@ inflateWithDict dict compressed = unsafePerformIO $ do
             Just z -> go (front . (:) z) x
 
 main :: IO ()
-main = hspecX $ do
+main = hspec $ do
     describe "inflate/deflate" $ do
         prop "decompress'" $ \lbs -> lbs == decompress' (compress lbs)
         prop "compress'" $ \lbs -> lbs == decompress (compress' lbs)
@@ -96,7 +96,7 @@ main = hspecX $ do
             raw <- L.readFile "LICENSE"
             deflated <- return $ deflateWithDict exampleDict raw
             inflated <- return $ inflateWithDict (S.drop 1 exampleDict) deflated
-            assertBool "is null" $ L.null inflated
+            inflated `shouldSatisfy` L.null
 
     describe "license" $ do
         it "single deflate" $ do
@@ -109,7 +109,7 @@ main = hspecX $ do
             gziped <- feedDeflate def license >>= go id
             gziped' <- go gziped $ finishDeflate def
             let raw' = L.fromChunks [license]
-            raw' @?= Gzip.decompress (L.fromChunks $ gziped' [])
+            raw' `shouldBe` Gzip.decompress (L.fromChunks $ gziped' [])
 
         it "single inflate" $ do
             let go front x = do
@@ -122,7 +122,7 @@ main = hspecX $ do
             popper <- feedInflate inf gziped
             ungziped <- go id popper
             final <- finishInflate inf
-            license @?= (S.concat $ ungziped [final])
+            license `shouldBe` (S.concat $ ungziped [final])
 
         it "multi deflate" $ do
             let go' inf front bs = feedDeflate inf bs >>= go front
@@ -135,7 +135,7 @@ main = hspecX $ do
             gziped <- foldM (go' def) id $ map S.singleton $ S.unpack license
             gziped' <- go gziped $ finishDeflate def
             let raw' = L.fromChunks [license]
-            raw' @?= (Gzip.decompress $ L.fromChunks $ gziped' [])
+            raw' `shouldBe` (Gzip.decompress $ L.fromChunks $ gziped' [])
 
         it "multi inflate" $ do
             let go' inf front bs = feedInflate inf bs >>= go front
@@ -149,7 +149,7 @@ main = hspecX $ do
             inf <- initInflate $ WindowBits 31
             ungziped' <- foldM (go' inf) id gziped'
             final <- finishInflate inf
-            license @?= (S.concat $ ungziped' [final])
+            license `shouldBe` (S.concat $ ungziped' [final])
 
     describe "lbs zlib" $ do
         prop "inflate" $ \lbs -> unsafePerformIO $ do
@@ -206,3 +206,45 @@ main = hspecX $ do
                 callback "finish" "" $ finishDeflate def
         it "zlib" $ helper defaultWindowBits
         it "gzip" $ helper $ WindowBits 31
+    describe "large raw #9" $ do
+        let size = fromIntegral $ LI.defaultChunkSize * 4 + 1
+            input = L.replicate size 10
+        it "compressing" $ do
+            output <- fmap Raw.decompress $ compressRaw input
+            L.all (== 10) output `shouldBe` True
+            L.length output `shouldBe` L.length input
+        it "decompressing" $ do
+            output <- decompressRaw $ Raw.compress input
+            L.all (== 10) output `shouldBe` True
+            L.length output `shouldBe` L.length input
+
+rawWindowBits :: WindowBits
+rawWindowBits = WindowBits (-15)
+
+decompressRaw :: L.ByteString -> IO L.ByteString
+decompressRaw gziped = do
+    inf <- initInflate rawWindowBits
+    ungziped <- foldM (go' inf) id $ L.toChunks gziped
+    final <- finishInflate inf
+    return $ L.fromChunks $ ungziped [final]
+  where
+    go' inf front bs = feedInflate inf bs >>= go front
+    go front x = do
+        y <- x
+        case y of
+            Nothing -> return front
+            Just z -> go (front . (:) z) x
+
+compressRaw :: L.ByteString -> IO L.ByteString
+compressRaw raw = do
+    def <- initDeflate 1 rawWindowBits
+    gziped <- foldM (go' def) id $ L.toChunks raw
+    gziped' <- go gziped $ finishDeflate def
+    return $ L.fromChunks $ gziped' []
+  where
+    go' def front bs = feedDeflate def bs >>= go front
+    go front x = do
+        y <- x
+        case y of
+            Nothing -> return front
+            Just z -> go (front . (:) z) x
